@@ -310,40 +310,53 @@ getPitchCep = function(frame,
 #'
 #' Internal soundgen function.
 #'
-#' Attempts to find F0 of a frame by calculating ratios of putative harmonics
-#' (frequency domain analysis, ~ modified BaNa algorithm). See Ba et al. (2012)
-#' "BaNa: A hybrid approach for noise resilient pitch detection." Statistical
-#' Signal Processing Workshop (SSP), 2012 IEEE.
+#' Attempts to find F0 of a frame by detecting several putative harmonics and
+#' either finding their highest common factor (specMethod = "commonFactor") or
+#' comparing their ratios (specMethod = "BaNa"). For the highest common factor
+#' method, see Howard & Angus (2017) "Acoustics and psychoacoustics" (section
+#' 3.2.1). For BaNa, see Ba et al. (2012) "BaNa: A hybrid approach for noise
+#' resilient pitch detection." Statistical Signal Processing Workshop (SSP),
+#' 2012 IEEE.
 #' @inheritParams analyzeFrame
 #' @inheritParams analyze
 #' @param bin the width of spectral bin in \code{frame}, Hz
 #' @param HNR harmonics-to-noise ratio returned by \code{\link{getPitchAutocor}}
+#' @param specMethod "commonFactor" = highest common factor of putative
+#'   harmonics, "BaNa" = ratio of putative harmonics
+#' @param specRatios for method = "commonFactor", the number of harmonics AND
+#'   integer fractions to consider
+#' @param specMerge pitch candidates within \code{specMerge} semitones are
+#'   merged with boosted certainty
 #' @param specThres voicing threshold (unitless, ~0 to 1)
 #' @param specPeak,specHNRslope when looking for putative harmonics in
 #'   the spectrum, the threshold for peak detection is calculated as
 #'   \code{specPeak * (1 - HNR * specHNRslope)}
 #' @param specSmooth the width of window for detecting peaks in the spectrum, Hz
-#' @param specMerge pitch candidates within \code{specMerge} semitones are
-#'   merged with boosted certainty
-#' @param specSinglePeakCert (0 to 1) if F0 is calculated based on a single
+#' @param specSinglePeakCert (0 to 1) if f0 is calculated based on a single
 #'   harmonic ratio (as opposed to several ratios converging on the same
 #'   candidate), its certainty is taken to be \code{specSinglePeakCert}
+#' @param nCands number of pitch candidates pre frame (specMethod =
+#'   "commonFactor" always returns a single candidate)
 #' @return Returns either NULL or a dataframe of pitch candidates.
 #' @keywords internal
 getPitchSpec = function(frame,
                         bin,
                         freqs,
+                        specMethod = c('commonFactor', 'BaNa')[1],
+                        specRatios,
                         specSmooth,
+                        specThres,
+                        specMerge,
+                        specPeak,
                         specHNRslope,
                         HNR = NULL,
-                        specThres,
-                        specPeak,
                         specSinglePeakCert,
                         pitchFloor,
                         pitchCeiling,
-                        specMerge,
                         nCands
 ) {
+  if (specMethod == 'commonFactor') nCands = 1
+  # nCands = 1, otherwise returns subh with the same apparent certainty
   pitchSpec_array = NULL
   n = length(frame)
   width = 2 * ceiling((specSmooth / bin + 1) * 20 / bin / 2) - 1 # to be always ~100 Hz,
@@ -358,6 +371,8 @@ getPitchSpec = function(frame,
   }
 
   # find peaks in the spectrum (hopefully harmonics)
+  # plot(freqs, frame, type = 'l')
+  # plot(freqs, frame, type = 'l', log = 'xy')
   temp = zoo::rollapply(zoo::as.zoo(frame),
                         width = width,
                         align = 'center',
@@ -387,55 +402,64 @@ getPitchSpec = function(frame,
   }
 
   if (nr == 1) {
-    if (specPeaks[1, 1] < pitchCeiling & specPeaks[1, 1] > pitchFloor) {
-      pitchSpec = specPeaks[1, 1]
+    if (specPeaks$freq < pitchCeiling & specPeaks$freq > pitchFloor) {
+      pitchSpec = specPeaks$freq
       pitchCert = specSinglePeakCert
       pitchSpec_array = data.frame(
         'pitchCand' = pitchSpec,
-        'pitchCert' = specSinglePeakCert,
+        'pitchCert' = pitchCert,
         'pitchSource' = 'spec',
         stringsAsFactors = FALSE,
         row.names = NULL
       )
     }
   } else if (nr > 1) {
-    # analyze five lowest harmonics
-    specPeaks = specPeaks[1:min(5, nrow(specPeaks)), ]
+    if (specMethod == 'commonFactor') {
+      # analyze specRatios lowest harmonics
+      specPeaks = specPeaks[1:min(specRatios, nrow(specPeaks)), ]
+      # Find all possible integer fractions of these putative harmonics.
+      # True pitch is the largest common factor
+      pitchCand = as.numeric(apply(specPeaks[, 'freq', drop = FALSE], 1,
+                                   function(x) x / (1:specRatios)))
+    } else if (specMethod == 'BaNa') {
+      # A modified version of BaNa algorithm follows
+      # analyze five lowest harmonics
+      specPeaks = specPeaks[1:min(5, nrow(specPeaks)), ]
 
-    # A modified version of BaNa algorithm follows
-    seq1 = 2:nrow(specPeaks)
-    seq2 = 1:(nrow(specPeaks) - 1)
-    n = length(seq1) * length(seq2)
-    temp = data.frame (
-      'harmonicA' = rep(0, n),
-      'harmonicB' = rep(0, n),
-      'AtoB_ratio' = rep(0, n)
-    )
-    counter = 1
-    for (i in seq1) {
-      for (j in seq2) {
-        temp$harmonicA[counter] = i
-        temp$harmonicB[counter] = j
-        # get ratios of discovered harmonics to each other
-        temp$AtoB_ratio[counter] = specPeaks$freq[i] / specPeaks$freq[j]
-        counter = counter + 1
+      seq1 = 2:nrow(specPeaks)
+      seq2 = 1:(nrow(specPeaks) - 1)
+      n = length(seq1) * length(seq2)
+      temp = data.frame (
+        'harmonicA' = rep(0, n),
+        'harmonicB' = rep(0, n),
+        'AtoB_ratio' = rep(0, n)
+      )
+      counter = 1
+      for (i in seq1) {
+        for (j in seq2) {
+          temp$harmonicA[counter] = i
+          temp$harmonicB[counter] = j
+          # get ratios of discovered harmonics to each other
+          temp$AtoB_ratio[counter] = specPeaks$freq[i] / specPeaks$freq[j]
+          counter = counter + 1
+        }
       }
-    }
-    temp = temp[temp$harmonicA > temp$harmonicB, ]
+      temp = temp[temp$harmonicA > temp$harmonicB, ]
 
-    pitchCand = numeric()
-    for (i in 1:nrow(temp)) {
-      # for each ratio that falls within the limits specified outside this
-      # function in a dataframe called "ratios", calculate the corresponding
-      # pitch. If several ratios suggest the same pitch, that's our best guess
-      divLow = BaNaRatios$divide_lower_by[temp$AtoB_ratio[i] > BaNaRatios$value_low &
-                                            temp$AtoB_ratio[i] < BaNaRatios$value_high]
-      pitchCand = c(pitchCand,
-                    as.numeric(specPeaks$freq[temp$harmonicB[i]] / divLow))
+      pitchCand = numeric()
+      for (i in 1:nrow(temp)) {
+        # for each ratio that falls within the limits specified outside this
+        # function in a dataframe called "ratios", calculate the corresponding
+        # pitch. If several ratios suggest the same pitch, that's our best guess
+        idx = which(temp$AtoB_ratio[i] > BaNaRatios$value_low &
+                      temp$AtoB_ratio[i] < BaNaRatios$value_high)
+        divLow = BaNaRatios$divide_lower_by[idx]
+        pitchCand = c(pitchCand,
+                      as.numeric(specPeaks$freq[temp$harmonicB[i]] / divLow))
+      }
+      # add pitchCand based on the most common distances between harmonics
+      # pitchCand = c(pitchCand, diff(specPeaks[,1]))
     }
-    # add pitchCand based on the most common distances between harmonics
-    # pitchCand = c(pitchCand, diff(specPeaks[,1]))
-
     pitchCand = sort(pitchCand[pitchCand > pitchFloor &
                                  pitchCand < pitchCeiling])
     if (length(pitchCand) > 0) {
@@ -462,6 +486,7 @@ getPitchSpec = function(frame,
           c = c + 1
         }
       }
+      pitchSpec_array = pitchSpec_array[pitchSpec_array$specAmplIdx > 1, ]
       pitchSpec_array$pitchCert = specSinglePeakCert +
         (1 / (1 + exp(-(pitchSpec_array$specAmplIdx - 1))) - 0.5) * 2 *
         (1 - specSinglePeakCert) # normalization. Visualization:
@@ -470,7 +495,7 @@ getPitchSpec = function(frame,
       # (1 - specSinglePeakCert)
       # plot(a, b, type = 'l')
       pitchSpec_array = pitchSpec_array[
-        order(pitchSpec_array$pitchCert, decreasing = TRUE),
+        order(pitchSpec_array$pitchCert, pitchSpec_array$pitchCand, decreasing = TRUE),
         c('pitchCand', 'pitchCert', 'pitchSource')
       ]
     }
@@ -597,3 +622,196 @@ getPitchHps = function(frame,
   }
   return(pitchHps_array)
 }
+
+
+#' Zero-crossing rate
+#'
+#' A less precise, but very quick method of pitch tracking based on measuring
+#' zero-crossing rate in low-pass-filtered audio. Recommended for processing
+#' long recordings with typical pitch values well below the first formant
+#' frequency, such as speech. Calling this function is considerably faster than
+#' using the same pitch-tracking method in \code{\link{analyze}}. Note that,
+#' unlike analyze(), it returns the times of individual zero crossings
+#' (hopefully corresponding to glottal cycles) instead of pitch values at fixed
+#' time intervals.
+#'
+#' Algorithm: the audio is bandpass-filtered from \code{pitchFloor} to \code{pitchCeiling}, and the timing of all zero crossings is saved. This is not enough, however, because unvoiced sounds like white noise also have plenty of zero crossings. Accordingly, an attempt is made to detect voiced segments (or steady musical tones, etc.) by looking for stable regions, with several zero-crossings at relatively regular intervals (see parameters \code{zcThres} and \code{zcWin}). Very quiet parts of audio are also treated as not having a pitch.
+#' @seealso \code{\link{analyze}}
+#'
+#' @inheritParams spectrogram
+#' @inheritParams segment
+#' @inheritParams analyze
+#' @param zcThres pitch candidates with certainty below this value are treated
+#'   as noise and set to NA (0 = anything goes, 1 = pitch must be perfectly
+#'   stable over \code{zcWin})
+#' @param zcWin certainty in pitch candidates depends on how stable pitch is
+#'   over \code{zcWin} glottal cycles (odd integer > 3)
+#' @param silence minimum root mean square (RMS) amplitude, below which pitch
+#'   candidates are set to NA (NULL = don't consider RMS amplitude)
+#' @param envWin window length for calculating RMS envelope, ms
+#'
+#' @return Returns a dataframe containing \describe{\item{time}{time stamps of
+#'   all zero crossings except the last one, after bandpass-filtering}
+#'   \item{pitch}{pitch calculated from the time between consecutive zero
+#'   crossings} \item{cert}{certainty in each pitch candidate calculated from
+#'   local pitch stability, 0 to 1}}
+#'
+#' @export
+#' @examples
+#' data(sheep, package = 'seewave')
+#' # spectrogram(sheep)
+#' zc = getPitchZc(sheep, pitchCeiling = 250)
+#' plot(zc$detailed[, c('time', 'pitch')], type = 'b')
+#'
+#' # Convert to a standard pitch contour sampled at regular time intervals:
+#' pitch = getSmoothContour(
+#'   anchors = data.frame(time = zc$detailed$time, value = zc$detailed$pitch),
+#'   len = 1000, NA_to_zero = FALSE, discontThres = 0)
+#' spectrogram(sheep, extraContour = pitch, ylim = c(0, 2))
+#'
+#' \dontrun{
+#' # process all files in a folder
+#' zc = getPitchZc('~/Downloads/temp')
+#' zc$summary
+#' }
+getPitchZc = function(x,
+                      samplingRate = NULL,
+                      scale = NULL,
+                      from = NULL,
+                      to = NULL,
+                      pitchFloor = 50,
+                      pitchCeiling = 400,
+                      zcThres = .1,
+                      zcWin = 5,
+                      silence = .04,
+                      envWin = 5,
+                      summaryFun = c('mean', 'sd'),
+                      reportEvery = NULL) {
+  # match args
+  myPars = as.list(environment())
+  # exclude some args
+  myPars = myPars[!names(myPars) %in%
+                    c('x', 'samplingRate', 'scale', 'from', 'to',
+                      'summaryFun', 'reportEvery')]
+  pa = processAudio(x,
+                    samplingRate = samplingRate,
+                    scale = scale,
+                    from = from,
+                    to = to,
+                    funToCall = '.getPitchZc',
+                    myPars = myPars,
+                    reportEvery = reportEvery
+  )
+
+  # prepare output
+  if (!is.null(summaryFun) && any(!is.na(summaryFun))) {
+    temp = vector('list', pa$input$n)
+    for (i in 1:pa$input$n) {
+      if (!pa$input$failed[i]) {
+        temp[[i]] = summarizeAnalyze(
+          pa$result[[i]] [, c('pitch', 'cert')],
+          summaryFun = summaryFun,
+          var_noSummary = NULL)
+      }
+    }
+    idx_failed = which(pa$input$failed)
+    if (length(idx_failed) > 0) {
+      idx_ok = which(!pa$input$failed)
+      if (length(idx_ok) > 0) {
+        filler = temp[[idx_ok[1]]] [1, ]
+        filler[1, ] = NA
+      } else {
+        stop('Failed to analyze any input')
+      }
+      for (i in idx_failed) temp[[i]] = filler
+    }
+    mysum_all = cbind(data.frame(file = pa$input$filenames_base),
+                      do.call('rbind', temp))
+  } else {
+    mysum_all = NULL
+  }
+  if (pa$input$n == 1) pa$result = pa$result[[1]]
+  invisible(list(
+    detailed = pa$result,
+    summary = mysum_all
+  ))
+}
+
+
+#' Zero-crossing rate per sound
+#'
+#' Internal soundgen function called by \code{\link{getPitchZc}}.
+#' @param audio a list returned by \code{readAudio}
+#' @inheritParams getPitchZc
+#' @param env precalculated envelope (when called internally by .analyze())
+#' @param certMethod method of calculating pitch certainty: 'autocor' =
+#'   autocorrelation of pitch estimates per zc over window (a measure of curve
+#'   smoothness), 'variab' = variability of pitch estimates per zc over window
+#' @keywords internal
+.getPitchZc = function(audio,
+                       pitchFloor,
+                       pitchCeiling,
+                       zcThres,
+                       zcWin = 5,
+                       silence = .04,
+                       env = NULL,
+                       envWin = 5,
+                       certMethod = c('autocor', 'variab')[2]) {
+  ## Find zero crossing in bandpass-filtered audio
+  audio_lowpass = .bandpass(audio, lwr = pitchFloor, upr = pitchCeiling)
+  zc = which(diff(sign(audio_lowpass)) == 2)
+  len_zc = length(zc)
+  if (len_zc < 2) return(data.frame(time = NA, pitch = NA, cert = NA)[-1, ])
+  len_pitch = len_zc - 1
+  pitch_zc = audio$samplingRate / diff(zc)
+  time_zc = zc[1:len_pitch] / audio$samplingRate * 1000
+  # plot(time_zc, pitch_zc, type = 'l')
+
+  ## Calculate zc pitch certainty over sliding window
+  win = seewave::ftwindow(wl = zcWin, wn = 'gaussian')
+  half_wl = floor(zcWin / 2)
+  if (certMethod == 'autocor') {
+    # Method 1: autocorrelation of pitch
+    pitch_padded = c(rep(NA, zcWin), pitch_zc, rep(NA, zcWin))
+    cert = rep(0, len_pitch)
+    for (i in 1:len_pitch) {
+      frame = pitch_padded[(i + zcWin - half_wl) : (i + zcWin + half_wl)]
+      A = frame[-zcWin]
+      B = frame[-1]
+      cert[i] = sum(A * B) / sqrt(sum(A ^ 2) * sum(B ^ 2))  # cor(frame[-length(frame)], frame[-1])
+      # df_temp = data.frame(x = 1:length(frame), y = frame)
+      # mod = summary(lm(y ~ x, df_temp))
+      # cert[i] = mod$r.squared
+    }
+    # plot(cert, type = 'l')
+  } else if (certMethod == 'variab') {
+    # Method 2: variability of pitch derivative
+    diff_pitch = c(0, abs(diff(log2(pitch_zc))) * 12)  # in semitones
+    diff_pitch_padded = c(rep(1e6, zcWin), diff_pitch, rep(1e6, zcWin))
+    deviation = cert = rep(0, len_pitch)
+    for (i in 1:len_pitch) {
+      frame = diff_pitch_padded[(i + zcWin - half_wl) : (i + zcWin + half_wl)]
+      deviation[i] = sum(frame * win)
+    }
+    # plot(deviation, type = 'l', ylim = c(0, 24))
+    cert = 1 / (deviation + 1)
+    # plot(cert, type = 'l')
+  }
+
+  ## Amplitude envelope - needed to unvoice very quiet sections
+  if (!is.null(silence)) {
+    if (is.null(env)) env = .getRMS(audio, windowLength = envWin, plot = FALSE)
+    env = resample(env, mult = len_pitch / length(env))
+    # plot(env, type = 'l')
+    cond_silence = env < silence
+  } else {
+    cond_silence = rep(FALSE, len_pitch)
+  }
+
+  ## Set quiet, unsteady, or impossibly low/high zc to NA
+  pitch_zc[cert < zcThres |
+             cond_silence |
+             pitch_zc < pitchFloor |
+             pitch_zc > pitchCeiling] = NA
+  return(data.frame(time = time_zc, pitch = pitch_zc, cert = cert))
+}
