@@ -938,39 +938,39 @@ server = function(input, output, session) {
 
   # HOTKEYS
   observeEvent(input$userPressedSmth, {
-    button_code = floor(input$userPressedSmth)
+    button_key = substr(input$userPressedSmth, 1, nchar(input$userPressedSmth) - 8)
     # see https://keycode.info/
-    if (button_code == 32) {                      # SPACEBAR (play / stop)
+    if (button_key == ' ') {                  # SPACEBAR (play / stop)
       if (myPars$play$on) stopPlay() else startPlay()
-    } else if (button_code == 37) {               # ARROW LEFT (scroll left)
+    } else if (button_key == 'ArrowLeft') {    # ARROW LEFT (scroll left)
       shiftFrame('left', step = myPars$scrollFactor)
-    } else if (button_code == 39) {               # ARROW RIGHT (scroll right)
+    } else if (button_key == 'ArrowRight') {    # ARROW RIGHT (scroll right)
       shiftFrame('right', step = myPars$scrollFactor)
-    } else if (button_code == 38) {               # ARROW UP (horizontal zoom-in)
+    } else if (button_key == 'ArrowUp') {       # ARROW UP (horizontal zoom-in)
       changeZoom(myPars$zoomFactor)
-    } else if (button_code == 83) {               # S (horizontal zoom to selection)
+    } else if (button_key %in% c('s', 'S')) {    # S (horizontal zoom to selection)
       zoomToSel()
-    } else if (button_code == 40) {               # ARROW DOWN (horizontal zoom-out)
+    } else if (button_key == 'ArrowDown') {   # ARROW DOWN (horizontal zoom-out)
       changeZoom(1 / myPars$zoomFactor)
-    } else if (button_code %in% c(61, 107)) {     # + (vertical zoom-in)
+    } else if (button_key == '+') {     # + (vertical zoom-in)
       changeZoom_freq(1 / myPars$zoomFactor_freq)
-    } else if (button_code %in% c(173, 109)) {    # - (vertical zoom-out)
+    } else if (button_key == '-') {    # - (vertical zoom-out)
       changeZoom_freq(myPars$zoomFactor_freq)
-    } else if (button_code == 13) {               # ENTER (next file)
+    } else if (button_key == 'PageDown') {    # PageDown (next file)
       nextFile()
-    } else if (button_code == 8) {                # BACKSPACE (previous file)
+    } else if (button_key == 'PageUp') {  # PageUp (previous file)
       lastFile()
-    } else if (button_code == 85) {               # U (unvoice)
+    } else if (button_key %in% c('u', 'U')) {  # U (unvoice)
       unvoiceSel()
-    } else if (button_code == 86) {               # V (voice)
+    } else if (button_key %in% c('v', 'V')) {  # V (voice)
       voiceSel()
-    } else if (button_code == 82) {               # R (raise by an octave)
+    } else if (button_key %in% c('r', 'R')) {  # R (raise by an octave)
       octaveUp()
-    } else if (button_code == 76) {               # L (lower by an octave)
+    } else if (button_key %in% c('l', 'L')) {  # L (lower by an octave)
       octaveDown()
-    } else if (button_code == 80) {               # P (set prior)
+    } else if (button_key %in% c('p', 'P')) {  # P (set prior)
       setPrior()
-    } else if (button_code == 68) {               # D (Draw pitch contour)
+    } else if (button_key %in% c('d', 'D')) {  # D (Draw pitch contour)
       obs_pitch()
     }
   })
@@ -1482,8 +1482,8 @@ server = function(input, output, session) {
   shinyBS::addTooltip(session, id='osc_maxPoints', title = 'The number of points to plot in the oscillogram (smaller = faster, but low resolution)', placement="below", trigger="hover", options = tooltip_options)
 
   # action buttons
-  shinyBS:::addTooltip(session, id='lastFile', title='Save and return to the previous file (BACKSPACE)', placement="right", trigger="hover", options = tooltip_options)
-  shinyBS:::addTooltip(session, id='nextFile', title='Save and proceed to the next file (ENTER)', placement="right", trigger="hover", options = tooltip_options)
+  shinyBS:::addTooltip(session, id='lastFile', title='Save and return to the previous file (PageUp)', placement="right", trigger="hover", options = tooltip_options)
+  shinyBS:::addTooltip(session, id='nextFile', title='Save and proceed to the next file (PageDown)', placement="right", trigger="hover", options = tooltip_options)
   shinyBS:::addTooltip(session, id='selection_play', title='Play selection (SPACEBAR)', placement="right", trigger="hover", options = tooltip_options)
   shinyBS::addTooltip(session, id='selection_unvoice', title = 'Unvoice selection (U)', placement="right", trigger="hover", options = tooltip_options)
   shinyBS::addTooltip(session, id='selection_voice', title = 'Voice selection (V) - obviouslly, pitch estimates may be totally incorrect', placement="right", trigger="hover", options = tooltip_options)
